#!/usr/bin/perl
#
# infoblox.pl
#
# Copyright (C) 2011 Michael McNamara
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Filename: /var/www/cgi-bin/infoblox.pl
#
# Purpose:  add/change/remove MAC addresses from MAC filtering on Infoblox
#
# Author:   Michael McNamara 
#
# Date:     October 24, 2011
#
# Changes:
#
#    October 24, 2011 (M.McNamara)
#       o  the beast is born had issues getting the Infoblox Perl libraries
#		to install on CentOS 5.7 due to a bug in perl-libwww-perl so
#		I built a CentOS 6.0 box that had a newer version of 
#		perl-libwww-perl so I didn't get bogged down. Once that was 
#		complete installing the Infoblox Perl libraries was pretty 
#		easy. I took some of the sample code from there API refrence
#		manually and started carving it up with a Perl CGI interface.
#    October 25, 2011 (M.McNamara)
#	o  we have a working product the basics work now we need to start cleaning
#		up the interface, adding error checking, adding logging, etc
#
#    October 27, 2011 (M.McNamara)
#	o  added basic form validation for the MAC address using a JavaScript call
#	o  added logic to search for the MAC address first before trying to add it.
#	o  added some CSS to help cleanup the look of the interface
#
#
# Program Flow: 
#

# Load Modules
use strict;
use warnings;
use CGI;
use CGI::Carp qw(fatalsToBrowser);
use Infoblox;

# Global Varibles
our $session;		# Infoblox Object
our $web;		# CGI Object
our $mac;		# Infoblox Object
our $fullurl;		# URL of script
our $macaddress;        # CGI Variable
our $description;       # CGI Variable
our $username;          # CGI Variable
our $DEBUG = 0;         # DEBUG FLAG for troubleshooting
our $action;		# What will we be doing
our ($sdate, $date, $time, $currentTime, $iTime);       # Time and Date

#
# THE VALUES BELOW SHOULD BE UPDATED FROM THE DEFAULTS
#
our $SERVER = "10.1.1.1";			# Infoblox Grid Master
our $USERID = "TestUser";			# Infoblox Username
our $PASSWD = "Test12345\$";			# Infoblox Password
our $MAC_ADDRESS = "99:88:77:66:55:44";		# USED FOR TESTING ONLY
our $MAC_FILTER = "IBFILTER";			# Filter used in Infoblox
our $TENYEARS = 31556926*10;			# Expiration time in seconds
our $OK_CHARS = 'a-zA-Z0-9 ,-:';		# Filter to sanitze input
our $COMPANY = "Acme Hospital";			# Company Name
our $CONTACT_NAME = 'Michael McNamara';		# Contact Name
our $CONTACT_EMAIL = 'user@somewhere.com';	# Contact Email
our $SCRIPT_URL = "http://web.acme.org"; 	# URL for CSS and Javascript

#########################################################################
#########################################################################
##  M  A  I  N     P  R  O  G  R  A  M
#########################################################################
#########################################################################

   # Let's intiailize our program
   &startup;

   # Let's output the HTML headers
   &html_header;

   if ( $action eq "add") {

      # If the script is being called with parameters let's process them
      &runjob;

   } elsif ($action eq "list" ) {

      # Let's retrieve and output the entire list of MAC addresses
      &listjob;

   } else {

      # If the script isn't being called with parameters let's display the form 
      &html_form();

   }

   # Let's output the HTML footers
   &html_footer;

exit 0;


#########################################################################
#########################################################################
###   E  N  D    P  R  O  G  R  A  M
#########################################################################
#########################################################################






###########################################################################
# Subroutine startup
#
# Purpose: perform all the initialization steps and procedures
###########################################################################
sub startup {

   # Inialize time and date
   $sdate = localtime;
   ($date, $time) = &get_time;
   $currentTime = time;

   # Let's create a new CGI object
   $web = new CGI;

   # Let's capture the full URL for future reference
   $fullurl = $web->url;

   # Disable IO Buffering
   $| = 1;
   
   # Let's get the userID from the Apache environment variables 
   $username = $ENV{'AUTHENTICATE_SAMACCOUNTNAME'};

   #if defined ($web->param('job')) { $action = $web->param('job'); }
   #if defined ($web->param('macaddress')) { $macaddress = $web->param('macaddress'); }
   #if defined ($web->param('description')) { $description = $web->param('description'); } 
   #if defined ($web->param('debug') { $DEBUG = $web->param('debug'); }

   $action = $web->param('job');
   $macaddress = $web->param('macaddress');
   $description = $web->param('description');
   $DEBUG = $web->param('debug');

   # Let's sanitize the input just in case
   $action =~ s/[^$OK_CHARS]//go; 
   $macaddress =~ s/[^$OK_CHARS]//go;
   $description =~ s/[^$OK_CHARS]//go;
   $username =~ s/[^$OK_CHARS]//go;
   $DEBUG =~ s/[^$OK_CHARS]//go;

} #end sub

###########################################################################
# Subroutine html_header 
#
# Purpose: output the HTML headers with the appropriate CSS and JavaScript
###########################################################################
sub html_header {

   # Let's output the HTML header using the CGI objects
   print $web->header;
   print $web->start_html( -title=>'Infoblox API - MAC registration',
		-style=>{-src=>'${SCRIPT_URL}/style.css'},
		-script=>{-type=>'Javascript', -src=>'${SCRIPT_URL}/javascript.js'}
	);

} #end sub

###########################################################################
# Subroutine html_footer
#
# Purpose: perform all the initialization steps and procedures
###########################################################################
sub html_footer {

   # Let's output the HTML footer and contact information
   print "If there are any questions please contact <a href=\"mailto:$CONTACT_EMAIL?subject=Infoblox API MAC Registration\">$CONTACT_NAME</a> thanks\!<br>\n";
   print "<span class=\"date\">$sdate</span>\n";
   print "</div>\n";
   print $web->end_html;

} #end sub

###########################################################################
# Subroutine app_header
#
# Purpose: output the application header for the webpage
###########################################################################
sub app_header {

   # Let's output the page header for the application
   print <<EOF;
  <div class="content">
  <h1>$COMPANY</h1>
  <h2>Infoblox API Interface - MAC registration</h2>
EOF

}
###########################################################################
# Subroutine app_continue
#
# Purpose: output to display the "Continue" button
###########################################################################
sub app_continue {

   # Let's output the continue button
   print "<br>\n";
   print "<button type=\"button\" class=\"continue\" onClick=\"window.location='$fullurl'\">Continue</button><br>\n";
   print "<br>\n";


}
###########################################################################
# Subroutine app_list_button
#
# Purpose: output to display the "List" button
###########################################################################
sub app_list_button {

   # Let's output the list button
   print "<br>\n";
   print "<button type=\"button\" class=\"continue\" onClick=\"window.location='$fullurl?job=list'\">List</button><br>\n";
   print "<br>\n";

}


###########################################################################
# Subroutine html_form
#
# Purpose: output the actual HTML form with all the parameters
###########################################################################
sub html_form {

   # The page header for the application
   &app_header;

   # I'm cheating here a little and just writing some HTML without
   # using the CGI objects which I shoudl probably try and keep to.
   print <<EOF;

   <form name="macform" method="POST" enctype="application/x-www-form-urlencoded" onsubmit="return validateForm();" action="$fullurl">
As part of our continuing efforts to secure the network we will be implementing MAC address registration on the Infoblox appliances. This requires that the MAC address of any device making a DHCP/BOOTP request be registered in the database prior to making any IP address requests. Any device that is not in the database will not receive an IP address from the DHCP server.<br>
  <br>
You can submit a MAC address below to have it registered in the database. The change is immediate. Please make sure to submit the MAC address in the following format; <i>xx:xx:xx:xx:xx:xx</i> and also post the asset name or description. Your userID will be logged as the person that added this device to the database so please make the descriptions useful.<br>
  <br>
  MAC Address&nbsp;&nbsp;<i>(Example: &nbsp;xx:xx:xx:xx:xx:xx)</i>
  <input type="text" name="macaddress" data-default="XX:XX:XX:XX:XX:XX" size="17" maxlength="17" class="required text clearField clearFieldActive"> 
  <br>
  Description&nbsp;&nbsp;<i>(Example: &nbsp;ML-MCNAMM - Michael McNamara)</i>
  <input type="text" name="description" data-default="Asset name or descriptor" size="50" maxlength="50" class="required text clearField clearFieldActive"> 
  <br>
  <input type="hidden" name="job" value="add"> 
  <input type="submit" value="Submit" name="B1"><input type="reset" value="Reset" name="B2">
  </form>

EOF

   &app_list_button;

} #end sub

###########################################################################
# Subroutine listjob 
#
# Purpose: list all MAC addresses registered in database 
###########################################################################
sub listjob {

   &app_header;

   &ib_session;

   &list_mac;

   &app_continue;

} #end sub
###########################################################################
# Subroutine runjob 
#
# Purpose: add a MAC address to the database
###########################################################################
sub runjob {

   &app_header;

   # DEBUG output
   print "MAC = $macaddress\n<br>";
   print "COMMENT = $description\n<br>";
   print "USERID = $username\n<br>";
   print "<br>\n";

   # We'll set the expiration time to 10 years in the future
   $iTime = $currentTime + $TENYEARS;

   &ib_session;

   &add_mac;

   &app_continue;

} #end sub

###########################################################################
# Subroutine ib_session
#
# Purpose: initialize the Infoblox session to the appliances
###########################################################################
sub ib_session {

   #Create a session to the Infoblox appliance
   $session = Infoblox::Session->new(
        master   => "$SERVER",          #appliance host ip
        username => "$USERID",          #appliance user login
        password => "$PASSWD"           #appliance password
   );

   unless ($session) {
        die("Construct session failed: ",
        Infoblox::status_code() . ":" . Infoblox::status_detail());
   }

   print "Session created successfully\n<br>";


} #end sub

###########################################################################
# Subroutine add_mac
#
# Purpose: search for the MAC address and then add if not existing 
###########################################################################
sub add_mac {

   # Let's replace the - with : in the MAC address
   $macaddress =~ s/-/:/g;

   # Let's convert MAC addres to all lowercase
   if (defined $macaddress) {
      $mac =~ tr/A-Z/a-z/;
   }

    print "Searching for MAC Address...\n<br>";

    my @retrieved_objs = $session->get(
	object => "Infoblox::DHCP::MAC",
	mac => "$macaddress",
	filter => "$MAC_FILTER",
    );

    my $object = $retrieved_objs[0];

    unless ($object) {

   	#Create a DHCP MAC object

   	my $mac = Infoblox::DHCP::MAC->new(
   	     "mac" => "$macaddress",
   	     "filter" => "$MAC_FILTER",
   	     "comment" => "$description",
   	     "username" => "$username",
   	     "authentication_time" => "$currentTime",
		"expiration_time" => "$iTime",
   	);

   	unless($mac) {
 		die("Construct MAC object failed: ",
       		Infoblox::status_code() . ":" . Infoblox::status_detail());
   	}

	print "MAC Address object created successfully\n<br><br>";

	#Search for DHCP MAC object first

	#Add the MAC Address object into the Infoblox appliance through a session
	$session->add($mac)
       	or die("Add MAC Address object failed: ",
        	$session->status_code() . ":" . $session->status_detail());

	print "<span class=\"macadded\">MAC Address object added successfully</span><br>\n";
    } else {

	print "<br><span class=\"macexists\">***MAC ADDRESS ALREADY EXISTS IN DATABASE ***</span><br>\n";

    }

} #end sub

###########################################################################
# Subroutine list_mac
#
# Purpose: output all the MAC addresses registered in the database
###########################################################################
sub list_mac {

    print "Listing MAC Addresses...\n<br>";

    my @list = $session->get(
        object => "Infoblox::DHCP::MAC",
        filter => "$MAC_FILTER",
    );

    my $nextobject = $list[0];

    #print "MAC = " . scalar($nextobject->mac()) . "<br>\n";
    #print "COMMENT = " . scalar($nextobject->comment()) . "<br>\n";
    #print "USERID = " . scalar($nextobject->username()) . "<br>\n";
    #print "EXPIRATION = " . scalar($nextobject->expiration_time()) . "<br>\n";

    print <<EOF;
    <br>
    <table>
    <tr>
    <th>MAC</th>
    <th>Description</th>
    <th>UserID</th>
    <th>Expiration</th>
    </tr>
EOF

    foreach my $test ( @list ) {

       print "<tr>";
       print "<td>" . scalar($test->mac()) . "</td>";
       print "<td>" . scalar($test->comment()) . "</td>\n";
       print "<td>" . scalar($test->username()) . "</td>\n";
       print "<td>" . scalar(localtime(scalar($test->expiration_time()))) . "</td>\n";
       print "</tr>";

       #sleep (1);

    }

    print "</table>";

} #end sub

########################################################################
# Subroutine get_time
#
# Purpose: calculate the time
########################################################################
sub get_time {
   # Declare Local Variables
   my ($sec, $min, $hour, $day, $mon, $year, $date, $time, $now);

   ($sec, $min, $hour, $day, $mon, $year) =  (localtime)[0,1,2,3,4,5];
   if ($sec < 10) { $sec = "0" . $sec }
   if ($min < 10) { $min = "0" . $min }
   if ($hour < 10) { $hour = "0" . $hour }
   $mon = $mon + 1;
   $year = $year + 1900;
   $date = $mon . "-" . $day . "-" . $year;
   $time = $hour . ":" . $min . ":" . $sec;
   $now = $date . " at " . $time;

   return ($date, $time);
} #end sub get_time

